﻿Imports NetFwTypeLib
Imports System.IO
Imports System.Net
Imports System.Text
Imports System.Text.Json

Public NotInheritable Class Webserver2

    Public Shared myLog As New List(Of String)  ' Log File because of assync task
    Public Shared DeviceCount As Integer        ' number of devices transmitted (for info)
    Public Shared listener As HttpListener

    'for test
    Public Shared JsonTest As String

    Public Shared Sub StartWebServer()
        listener = Nothing
        Log("Restarting Webserver...")
        Dim prefix = "http://*:" & My.Settings.ListenPort.ToString & "/"
        listener = New HttpListener()
        listener.Prefixes.Add(prefix)
        Log("Listening on Port : " & My.Settings.ListenPort.ToString)
        listener.Start()
        listener.BeginGetContext(New AsyncCallback(AddressOf NewContext), listener)
    End Sub

    Public Shared Sub StopWebserver()
        Log("Stopping Webserver...")
        If listener.IsListening Then
            listener.Stop()
        End If
    End Sub


    Public Shared Sub NewContext(iar As IAsyncResult)
        Dim m = DirectCast(iar.AsyncState, HttpListener)

        If m.IsListening Then 'False wenn listener.Stop aufgerufen wurde
            Dim context As HttpListenerContext = m.EndGetContext(iar)
            ProcessRequest(context)

            'Kontext verwenden
            'Dim request As HttpListenerRequest = context.Request
            'Dim req As System.IO.Stream = request.InputStream
            'Dim enc As System.Text.Encoding = request.ContentEncoding
            'Dim read As System.IO.StreamReader = New System.IO.StreamReader(req, enc)
            'Dim s As String = read.ReadToEnd
            'test.Add(Uri.UnescapeDataString(s))
            'req.Close()
            'read.Close()
            'Dim response As HttpListenerResponse = context.Response
            'Dim responseString As String = "1"
            'Dim buffer As Byte() = System.Text.Encoding.UTF8.GetBytes(responseString)
            'response.ContentLength64 = buffer.Length
            'Dim output As System.IO.Stream = response.OutputStream
            'output.Write(buffer, 0, buffer.Length)
            'output.Close()
            listener.BeginGetContext(New AsyncCallback(AddressOf NewContext), listener)
        Else
            m.Close() 'Schließen wenns die letzte Verbindung war
        End If
    End Sub


    Private Shared Sub ProcessRequest(ByVal context As HttpListenerContext)
        Log("New Request detected ...")

        Dim body = New StreamReader(context.Request.InputStream).ReadToEnd()

        ' Dim ar As New cAlexaRequest
        ' Dim uu = Type.GetType("AlexaDeviceGenerator.cAlexaRequest")

        Dim ar As cAlexaRequest = JsonSerializer.Deserialize(body, Type.GetType("AlexaDeviceGenerator.cAlexaRequest"))

        If ar.directive.header.namespace = "Alexa.Discovery" Then
            Log("Alexa tries to discover devices")
            Dim mJson As String = BuildResponseJson()
            Dim b As Byte() = Encoding.UTF8.GetBytes(mJson)
            context.Response.StatusCode = 200
            context.Response.KeepAlive = False
            context.Response.ContentLength64 = b.Length
            Dim output = context.Response.OutputStream
            output.Write(b, 0, b.Length)
            Log("Responding with " & DeviceCount & " device(s)")
            Log("------------------------------------ ")
            context.Response.Close()
        End If

    End Sub


    ' ************************************************************************************************
    ' ************************************************************************************************
    ' Build the JSON Response to Alexa from a clone of ResponseObj
    ' ************************************************************************************************
    ' ************************************************************************************************

    Public Shared Function BuildNiceResponseJson() As String
        Dim ResponseObj As New Cls_ResponseObjects
        ResponseObj.event = New Cls_event
        ResponseObj.event.header = New Cls_Header
        ResponseObj.event.payload = New Cls_Payload
        DeviceCount = 0

        For Each o As Object In myResponses.event.payload.endpoints
            If o.enabled = True Then
                DeviceCount = DeviceCount + 1
                ResponseObj.event.payload.AddEndpoint(o)
            End If
        Next
        ResponseObj.GenerateMessageId()
        Return ResponseObj.CreateJSON(NiceJSON)
    End Function

    Public Shared Function BuildResponseJson() As String
        '       Return JsonTest
        Dim ResponseObj As New Cls_ResponseObjects
        ResponseObj.event = New Cls_event
        ResponseObj.event.header = New Cls_Header
        ResponseObj.event.payload = New Cls_Payload
        DeviceCount = 0

        For Each o As Object In myResponses.event.payload.endpoints
            If o.enabled = True Then
                DeviceCount = DeviceCount + 1
                ResponseObj.event.payload.AddEndpoint(o)
            End If
        Next
        ResponseObj.GenerateMessageId()
        Return ResponseObj.CreateJSON(TransferJSON)
    End Function


    ' Put to to List
    Public Shared Sub Log(s As String)
        myLog.Add(TimeOfDay & " :-> " & s)
        '    myLog.Pop = myLog & TimeOfDay & " " & s & vbCrLf
        Debug.Write(s & vbCrLf)
    End Sub


    ' ************************************************************************************************
    ' ************************************************************************************************
    ' Open Firewall
    ' ************************************************************************************************
    ' ************************************************************************************************

    Public Shared Sub OpenFirewall()
        FWRule(NET_FW_RULE_DIRECTION_.NET_FW_RULE_DIR_OUT, NET_FW_ACTION_.NET_FW_ACTION_ALLOW, "1")
        FWRule(NET_FW_RULE_DIRECTION_.NET_FW_RULE_DIR_IN, NET_FW_ACTION_.NET_FW_ACTION_ALLOW, "1")
    End Sub


    '''<summary>
    ''' Adds Or removes a firewall rule.
    ''' </summary>
    ''' <param name="d">The affected connection type.</param>
    ''' <param name="fwaction">Rule action.</param>
    ''' <param name="action">"Add (1) or 
    ''' remove (0) the specified rule."</param>
    Public Shared Sub FWRule(ByVal d As NET_FW_RULE_DIRECTION_, ByVal fwaction As NET_FW_ACTION_, ByVal action As String)
        Try
            Dim firewallRule As INetFwRule = CType(Activator.CreateInstance(Type.GetTypeFromProgID("HNetCfg.FWRule")), INetFwRule)
            firewallRule.Action = fwaction
            firewallRule.Enabled = True
            firewallRule.InterfaceTypes = "All"
            firewallRule.ApplicationName = Application.ExecutablePath 'path
            firewallRule.Name = Application.ProductName ' "CSwitch: " & path.GetFileName(path)
            Dim firewallPolicy As INetFwPolicy2 = CType(Activator.CreateInstance(Type.GetTypeFromProgID("HNetCfg.FwPolicy2")), INetFwPolicy2)
            firewallRule.Direction = d

            If action = "1" Then
                firewallPolicy.Rules.Add(firewallRule)
            Else
                firewallPolicy.Rules.Remove(firewallRule.Name)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "ERROR")
        End Try
    End Sub
End Class



' ************************************************************************************************
' ************************************************************************************************
' Alexa Request Object, used to deserialize JSON
' ************************************************************************************************
' ************************************************************************************************

<Serializable()>
Public Class cAlexaRequest
    Public Property directive As cdirective
End Class

<Serializable()>
Public Class cdirective
    Public Property header As cHeader
    Public Property payload As cpayload
End Class

<Serializable()>
Public Class cHeader
    Public Property [namespace] As String
    Public Property instance As String
    Public Property name As String
    Public Property correlationToken As String
    Public Property payloadVersion As String
    Public Property messageId As String
End Class

<Serializable()>
Public Class cpayload
    Public Property scope As cscope
End Class

Public Class cscope
    Public Property [type] As String
    Public Property token As String
End Class

